# api 请求的统一封装


使用这个统一封装的请求可能会帮你省略一些业务逻辑代码。
## 使用

> 请求方法建议都以 Api 结尾

基本使用

```js
import { get, post } from "@/utils/fetch/index"

// 后端返回完整响应 { code:0,data:[....] } , 调用此方法会直接拿到 data 这一层，无需再解构一层
export const getListApi = (params) => get("/api/budget/report/detail", params)
```

### 设置当前接口 获取后端返回完整响应体

通过 `isReturnNativeResponse`  进行配置

场景：由于正常情况下我们只需要获取响应体中的 `data` 信息，而无需再判断业务 `code` 是否返回正常（因为业务code 返回异常在全局就进行了拦截）。  
但是不能保证后端同学由于某种场景 又重新定义了一个业务code ， 并且这个业务code = n 的时候 前端要做某些特定的处理。  
这个时候 就需要我们拿到完整的响应体 然后再去做一些额外的处理。

```js
// 返回后端接口完整响应 ， 设置 isReturnNativeResponse，可拿到 { code:0,data:[....] } 这一层
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    isReturnNativeResponse: true,
  })
```

### 设置当前接口 业务code非正常的情况下，的是否需要展示弹框

场景：一般情况非正常业务code ， 请求全局拦截器 会弹出一个 Message 弹框， 有些时候 不想使用这个弹框，而是触发一个别的逻辑 。
这个时候 我就可以 通过 `ifShowErrorModal` 来设置当前接口 如果返回非正常 code 也不做弹框处理，从而实现自己的页面逻辑。 

```js
// 如果后端业务code非正常状态，设置ifShowErrorModal 可不开启全局错误弹框，配合 isReturnNativeResponse 可根据当前业务逻辑自行处理。
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    ifShowErrorModal: false,
  })
```

### 设置当前接口 业务code非正常的情况 弹框的类型 

通过 `errorMessageMode` 进行配置。  

弹框一共有两种类型，一种是饿了么的 Message.error 另一种是 MessageBox , 可通过过设置 errorMessageMode 自行选择

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    errorMessageMode: "messageBox",
  })
```

### 设置重复请求，忽略上一次请求

通过 `ignoreCancelToken` 进行配置。  

是否需要忽略请求，比如我们获取报表数据查询较慢的时候，用户通过点击筛选项重新发送了一个请求，我们要关闭上一个请求。避免页面多次获取请求导致的重排。 通过 `ignoreCancelToken` 进行配置

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    ignoreCancelToken: true,
  })
```

### 设置当前接口 是否需要token 

默认所有接口都是带有token的，请求发送之会在请求拦截器 先进行判断本地存储是否存在token，如果本地存储没有token的话会自动走到登录的逻辑。
但是有些场景 我们调用一些 三方平台接口，可能不需要携带token 不走前端token的校验， 这个时候 我们可以通过 `withToken` 进行配置

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    withToken: true,
  })
```

### 设置当前接口 数据缓存 

你可以通过设置 `isCache` 来启用缓存功能。当缓存启用后 会把当前请求的结果缓存下来，等到下次该组件初始化时，亦或者另外一个设置了同样 `isCache` 的请求初始化时，  
如果有缓存，则会优先返回缓存的数据。

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    isCache: true,
  })
```

### 设置当前接口 数据缓存 并且自定义设置缓存时效 

设置当前接口缓存时效, 如果设置 `cacheTime` ( 单位毫秒 ) , 可不用设置 `isCache` 字段 

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    // 缓存 60s 
    cahceTime: 1000 * 60 
  })
```

### 设置当前接口 返回的数组 做字段映射

前端一些下拉选项，接收的值都是 [{label:"xxx",value:123}] 这种格式的数据，但时候 后端返回数据的时候 可能是 [{name:"xxx",id:123}] 这种格式。  
这个时候 就需要将 name 转换成 `label` , id 转换成 `value` 。 这种场景还是比较常见的 ， 所以提供了一个便捷属性 `field` 。  
只需要提供配置 field 参数，并填写一个字符串 就可以把对应的 `name` 转换成 `label` 。

```js
export const getListApi = (params) =>
  get("/api/budget/report/detail", params, {
    field: "name as label,id as value"
  })
```

清除

## requestOptions Props

| 参数                   | 说明                                                        | 类型    | 可选值       | 默认值    |
| ---------------------- | ----------------------------------------------------------- | ------- | ------------ | --------- |
| isReturnNativeResponse | 是否要返回最外层完整响应                                    | Boolean | true         | false     |
| ifShowErrorModal       | 业务 code 非成功状态是否需要弹框                            | Boolean | false        | true      |
| errorMessageMode       | 业务 code 非成功状态，错误弹框类型                          | string  | 'messageBox' | 'message' |
| ignoreCancelToken      | 是否忽略重复请求                                            | Boolean | true         | false     |
| withToken              | 是否需要携带 token                                          | Boolean | false        | true      |
| isRemoveEmptyParams    | 是否去掉参数为空值的入参，只限 POST 请求                    | Boolean | true         | false     |
| isCache                | 是否需要缓存，如果为 true 默认缓存 60 分钟                  | Boolean | true         | false     |
| cacheTime              | 数据缓存时间,单位毫秒，如果设置此值无需在设置 isCache,      | Number  | -            | -         |
| field                  | 针对数据选项 设置 字段映射 例如 "name as label,id as value" | String  | -            | -         |
